@@ -8,6 +8,7 @@
 #![recursion_limit="256"]
 
 #![deny(rust_2018_idioms)]
+#![deny(internal)]
 
 #[macro_use] extern crate rustc;
 #[allow(unused_extern_crates)]
@@ -21,7 +22,7 @@ mod persist;
 
 pub use assert_dep_graph::assert_dep_graph;
 pub use persist::dep_graph_tcx_init;
-pub use persist::load_dep_graph;
+pub use persist::{DepGraphFuture, load_dep_graph};
 pub use persist::load_query_result_cache;
 pub use persist::LoadResult;
 pub use persist::copy_cgu_workproducts_to_incr_comp_cache_dir;
