@@ -1,3 +1,4 @@
+// compile-pass
 // edition:2018
 // compile-flags:--extern edition_imports_2015
 // aux-build:edition-imports-2015.rs
@@ -12,8 +13,7 @@ mod check {
     pub struct Ambiguous {}
 
     fn check() {
-        edition_imports_2015::gen_ambiguous!(); //~ ERROR `Ambiguous` is ambiguous
-                                                //~| ERROR `edition_imports_2015` is ambiguous
+        edition_imports_2015::gen_ambiguous!(); // OK
     }
 }
 
