@@ -1,13 +1,3 @@
-// Copyright 2015 The Rust Project Developers. See the COPYRIGHT
-// file at the top-level directory of this distribution and at
-// http://rust-lang.org/COPYRIGHT.
-//
-// Licensed under the Apache License, Version 2.0 <LICENSE-APACHE or
-// http://www.apache.org/licenses/LICENSE-2.0> or the MIT license
-// <LICENSE-MIT or http://opensource.org/licenses/MIT>, at your
-// option. This file may not be copied, modified, or distributed
-// except according to those terms.
-
 // Demonstrate that having a trait bound causes dropck to reject code
 // that might indirectly access previously dropped value.
 //
@@ -41,8 +31,7 @@ fn main() {
 
     last_dropped = ScribbleOnDrop(format!("last"));
     first_dropped = ScribbleOnDrop(format!("first"));
-    foo0 = Foo(0, &last_dropped);
-    //~^ ERROR `last_dropped` does not live long enough
+    foo0 = Foo(0, &last_dropped); // OK
     foo1 = Foo(1, &first_dropped);
     //~^ ERROR `first_dropped` does not live long enough
 
